
package org.mule.module.linkedin.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.linkedin.config.GetConnectionsByUrlMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class GetConnectionsByUrlDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public GetConnectionsByUrlDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(GetConnectionsByUrlMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accessToken")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessToken")))) {
            builder.addPropertyValue("accessToken", element.getAttribute("accessToken"));
        }
        if ((element.getAttribute("accessTokenSecret")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessTokenSecret")))) {
            builder.addPropertyValue("accessTokenSecret", element.getAttribute("accessTokenSecret"));
        }
        if ((element.getAttribute("url")!= null)&&(!StringUtils.isBlank(element.getAttribute("url")))) {
            builder.addPropertyValue("url", element.getAttribute("url"));
        }
        Element profileFieldsListElement = null;
        profileFieldsListElement = DomUtils.getChildElementByTagName(element, "profile-fields");
        List<Element> profileFieldsListChilds = null;
        if (profileFieldsListElement!= null) {
            String profileFieldsRef = profileFieldsListElement.getAttribute("ref");
            if ((profileFieldsRef!= null)&&(!StringUtils.isBlank(profileFieldsRef))) {
                if ((!profileFieldsRef.startsWith(patternInfo.getPrefix()))&&(!profileFieldsRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("profileFields", new RuntimeBeanReference(profileFieldsRef));
                } else {
                    builder.addPropertyValue("profileFields", profileFieldsRef);
                }
            } else {
                ManagedList profileFields = new ManagedList();
                profileFieldsListChilds = DomUtils.getChildElementsByTagName(profileFieldsListElement, "profile-field");
                if (profileFieldsListChilds!= null) {
                    for (Element profileFieldsChild: profileFieldsListChilds) {
                        String valueRef = profileFieldsChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            profileFields.add(new RuntimeBeanReference(valueRef));
                        } else {
                            profileFields.add(profileFieldsChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("profileFields", profileFields);
            }
        }
        if ((element.getAttribute("start")!= null)&&(!StringUtils.isBlank(element.getAttribute("start")))) {
            builder.addPropertyValue("start", element.getAttribute("start"));
        }
        if ((element.getAttribute("count")!= null)&&(!StringUtils.isBlank(element.getAttribute("count")))) {
            builder.addPropertyValue("count", element.getAttribute("count"));
        }
        if ((element.getAttribute("modificationDate")!= null)&&(!StringUtils.isBlank(element.getAttribute("modificationDate")))) {
            builder.addPropertyValue("modificationDate", element.getAttribute("modificationDate"));
        }
        if (element.hasAttribute("modificationType")) {
            builder.addPropertyValue("modificationType", element.getAttribute("modificationType"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
