
package org.mule.module.linkedin.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.linkedin.config.SendMessageMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class SendMessageDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public SendMessageDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(SendMessageMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accessToken")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessToken")))) {
            builder.addPropertyValue("accessToken", element.getAttribute("accessToken"));
        }
        if ((element.getAttribute("accessTokenSecret")!= null)&&(!StringUtils.isBlank(element.getAttribute("accessTokenSecret")))) {
            builder.addPropertyValue("accessTokenSecret", element.getAttribute("accessTokenSecret"));
        }
        Element recepientIdsListElement = null;
        recepientIdsListElement = DomUtils.getChildElementByTagName(element, "recepient-ids");
        List<Element> recepientIdsListChilds = null;
        if (recepientIdsListElement!= null) {
            String recepientIdsRef = recepientIdsListElement.getAttribute("ref");
            if ((recepientIdsRef!= null)&&(!StringUtils.isBlank(recepientIdsRef))) {
                if ((!recepientIdsRef.startsWith(patternInfo.getPrefix()))&&(!recepientIdsRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("recepientIds", new RuntimeBeanReference(recepientIdsRef));
                } else {
                    builder.addPropertyValue("recepientIds", recepientIdsRef);
                }
            } else {
                ManagedList recepientIds = new ManagedList();
                recepientIdsListChilds = DomUtils.getChildElementsByTagName(recepientIdsListElement, "recepient-id");
                if (recepientIdsListChilds!= null) {
                    for (Element recepientIdsChild: recepientIdsListChilds) {
                        String valueRef = recepientIdsChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            recepientIds.add(new RuntimeBeanReference(valueRef));
                        } else {
                            recepientIds.add(recepientIdsChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("recepientIds", recepientIds);
            }
        }
        if ((element.getAttribute("subject")!= null)&&(!StringUtils.isBlank(element.getAttribute("subject")))) {
            builder.addPropertyValue("subject", element.getAttribute("subject"));
        }
        if ((element.getAttribute("message")!= null)&&(!StringUtils.isBlank(element.getAttribute("message")))) {
            builder.addPropertyValue("message", element.getAttribute("message"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
